unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry, ShlObj, ActiveX;

type
  TMainForm = class(TForm)
    bthBrowse: TButton;
    edtComputerName: TEdit;
    Memo: TMemo;
    btnGet: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bthBrowseClick(Sender: TObject);
    procedure btnGetClick(Sender: TObject);
  private
    { Private declarations }
    FReg: TRegistry;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

const
  INIT_KEY     = 'Software\NetUtils\Rtod\';
  NERR_Success = 0;

resourcestring
  STop  = 'Top';
  SLeft = 'Left';

type
  NET_API_STATUS = DWORD;

  PTimeOfDayInfo = ^TTimeOfDayInfo;
  TTimeOfDayInfo = packed record
    tod_elapsedt: DWORD;
    tod_msecs: DWORD;
    tod_hours: DWORD;
    tod_mins: DWORD;
    tod_secs: DWORD;
    tod_hunds: DWORD;
    tod_timezone: Longint;
    tod_tinterval: DWORD;
    tod_day: DWORD;
    tod_month: DWORD;
    tod_year: DWORD;
    tod_weekday: DWORD;
  end;

function NetRemoteTOD(UncServerName: LPCWSTR; BufferPtr: PBYTE): NET_API_STATUS; stdcall;
  external 'netapi32.dll' name 'NetRemoteTOD';

function NetApiBufferFree(Buffer: Pointer): NET_API_STATUS; stdcall;
  external 'netapi32.dll' name 'NetApiBufferFree';

function SelectComputer(const Caption: string; out Computer: string): Boolean;
var
   BrowseInfo: TBrowseInfo;
   ShellMalloc: IMalloc;
   RootItemIDList, ItemIDList: PItemIDList;
   Buffer: PChar;
begin
     Result := False;
     Computer := '';
     FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
     if (ShGetMalloc(ShellMalloc) = S_OK) and (ShellMalloc <> nil) then
     begin
          Buffer := ShellMalloc.Alloc(MAX_COMPUTERNAME_LENGTH + 1);
          try
             SHGetSpecialFolderLocation(0, CSIDL_NETWORK, RootItemIDList);
             with BrowseInfo do
             begin
                  hwndOwner := Application.Handle;
                  pidlRoot := RootItemIDList;
                  pszDisplayName := Buffer;
                  lpszTitle := PChar(Caption);
                  ulFlags := BIF_BROWSEFORCOMPUTER;
             end;
             ItemIDList := ShBrowseForFolder(BrowseInfo);
             Result := ItemIDList <> nil;
             if Result then
             begin
                  ShellMalloc.Free(ItemIDList);
                  Computer := '\\' + AnsiUpperCase(Buffer);
             end;
          finally
             ShellMalloc.Free(RootItemIDList);
             ShellMalloc.Free(Buffer);
          end;
     end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
   ComputerName: array[0..MAX_COMPUTERNAME_LENGTH] of Char;
   MaxComputerName: Cardinal;
begin
     FReg := TRegistry.Create;
     with FReg do if KeyExists(INIT_KEY) then
     begin
          OpenKey(INIT_KEY, False);
          if ValueExists(STop) then Top := ReadInteger(STop);
          if ValueExists(SLeft) then Left := ReadInteger(SLeft);
          CloseKey;
     end;
     MaxComputerName := SizeOf(ComputerName);
     GetComputerName(ComputerName, MaxComputerName);
     edtComputerName.Text := '\\' + ComputerName;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
     with FReg do
     begin
          OpenKey(INIT_KEY, True);
          WriteInteger(STop, Top);
          WriteInteger(SLeft, Left);
          CloseKey;
     end;
end;

procedure TMainForm.bthBrowseClick(Sender: TObject);
var
   s: string;
begin
     SelectComputer('Select computer', s);
     if s <> '' then edtComputerName.Text := s;
end;

procedure TMainForm.btnGetClick(Sender: TObject);
var
   TimeOfDayInfo: PTimeOfDayInfo;
   ServerName: array[0..255] of WideChar;
   dwRetValue, dwDays, dwSecs: DWORD;
   GMTTime: TSystemTime;
   CurTime, UpTime: TDateTime;
   ts: TTimeStamp;
begin
     StringToWideChar(edtComputerName.Text, @ServerName, SizeOf(ServerName));
     dwRetValue := NetRemoteTOD(@ServerName, PBYTE(@TimeOfDayInfo));
     if dwRetValue <> NERR_Success then
        raise Exception.Create(SysErrorMessage(dwRetValue));
     with TimeOfDayInfo^ do
     begin
          FillChar(GMTTime, SizeOf(GMTTime), 0);
          with GMTTime do
          begin
               wYear := tod_year;
               wMonth := tod_month;
               wDayOfWeek := tod_weekday;
               wDay := tod_day;
               wHour := tod_hours;
               wMinute := tod_mins;
               wSecond := tod_secs;
               wMilliseconds := tod_hunds;
          end;
          CurTime := SystemTimeToDateTime(GMTTime);
          if tod_timezone <> -1 then
             CurTime := CurTime + ((1/24/60) * -(tod_timezone));
          dwDays := 0;
          dwSecs := tod_msecs div 1000;
          if dwSecs >= SecsPerDay then
          begin
               dwDays := dwSecs div SecsPerDay;
               dwSecs := dwSecs mod SecsPerDay;
          end;
          ts.Time := dwSecs * 1000;
          UpTime := TimeStampToDateTime(ts);
          with Memo.Lines do
          begin
               Add(StringOfChar(' ', 40) +  '*** ' + edtComputerName.Text + ' ***');
               Add(Format('System UpTime: %u days, %s hours, %s minutes, %s seconds',
                         [dwDays, FormatDateTime('h', UpTime),
                          FormatDateTime('n', UpTime), FormatDateTime('s', UpTime)]));
               Add(Format('The current date is: %s', [FormatDateTime(ShortDateFormat, CurTime)]));
               Add(Format('The current time is: %s', [FormatDateTime(LongTimeFormat, CurTime)]));
               EndUpdate;
          end;

     end;
     NetApiBufferFree(TimeOfDayInfo);
end;

end.
